<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://kartik.webfixfast.com/
 * @since      1.0.0
 *
 * @package    Easy_Accordion_Posts
 * @subpackage Easy_Accordion_Posts/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Easy_Accordion_Posts
 * @subpackage Easy_Accordion_Posts/public
 * @author     WordPressaHolic <wordpressaholic@gmail.com>
 */
class Easy_Accordion_Posts_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The count of the total numbe of eap shortcodes on the page.
	 * Used to provide temporary classes to the grids
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $count    The total count of shortcodes on page.
	 */
	private $count;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->count = 0;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Easy_Accordion_Posts_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Easy_Accordion_Posts_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/easy-accordion-posts-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Easy_Accordion_Posts_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Easy_Accordion_Posts_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/easy-accordion-posts-public.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Register the [eap] main shortcode
	 *
	 * @since    1.0.0
	 */
	public function shortcode( $atts, $content = null ) {

		if( empty( $content ) ) return; // empty shortcode
		$content = html_entity_decode( $content );
		$content = str_replace(array('”', '“', '″'), '"', $content);
		$args = json_decode( $content, true );
		$atts = shortcode_atts( array(
		    'query' => false,
				'related_posts'=> false,
				'related_tax' => false,
				'align' => 'center',
				'id' => null,
				'no_results' => __( 'Sorry! No Results.', 'easy-accordion-posts' ),
		), $atts );

		++$this->count; // this field is used to track the css class and pagination id of the current grid

		if( $atts[ 'query' ] ){
			$atts[ 'query' ] = ltrim( $atts[ 'query' ], "$" );
			foreach( $GLOBALS as $var_name => $value ){
				if ($var_name === $atts[ 'query' ]) {
					$query_args = $value;
				}
			}
		}

		// database field should have vals
		if( empty( $args['database'] ) ){
			$args['database'] = array( 'post_type'=> array( ) );
		}
		if( empty( $args['database'][ 'post_type' ] ) )
			$args['database'][ 'post_type' ] = array( 'post'=> array( 'taxonomy'=> array( ) ) );

		// tax filter
		if( ! empty( $atts['id'] ) ){
			$prefix = $atts['id'] .'_tax_';
			$prefixlen = strlen( $prefix );
			$cleared = false;
			foreach( $_GET as $key=> $val ){
				$test = substr( $key, 0, $prefixlen );
				if( $test === $prefix ){ // found a match : means we have a tax:term_id filter set for this grid

					if( ! $cleared ){ // need to empty db tax one time
						foreach( $args['database'][ 'post_type' ] as $post_type_name => &$post_type_arr ){
							$post_type_arr[ 'taxonomy' ] = array( );
						}
						$cleared  = true;
					}

					$keylen = strlen( $key );
					$tax = substr( $key, $prefixlen, $keylen );
					$term_id = &$val;
					foreach( $args['database'][ 'post_type' ] as $post_type_name => &$post_type_arr ){
						if( ! isset( $post_type_arr[ 'taxonomy' ] ) ) $post_type_arr[ 'taxonomy' ] = array( );

						$post_type_arr[ 'taxonomy' ][ $tax ] = array(
							'terms'=> array(
								$term_id => array( )
							)
						);
					}
				}
			}
		}

		// build query args
		if( empty( $query_args ) ){
			$query_args = array(
				'posts_per_page' => ! empty( $args['query']['posts_per_page'] ) ? $args['query']['posts_per_page'] : 10,
				'post_type' => array(),
				'tax_query' => array(
					'relation' => 'AND',
				),
			);

			// pagination
			if( ! empty( $_GET['eap_'. $this->count .'_paged'] ) )
				$query_args[ 'paged' ] = $_GET['eap_'. $this->count .'_paged'];
			else if( ! empty( $_GET['eap_paged_'. $atts['id']] ) )
				$query_args[ 'paged' ] = $_GET['eap_paged_'. $atts['id']];

			// search
			if( ! empty( $_GET[ 'eap_search_'. $atts['id'] ] ) )
				$query_args[ 's' ] = $_GET[ 'eap_search_'. $atts['id'] ];

			// related posts starts
			if( $atts['related_posts'] ){
				global $post;
				$post_type = $post->post_type;
				$args['database'] = array(
					'post_type'=>array(
						$post_type => array(
							'taxonomy'=> array( )
						)
					)
				);

				if( $atts[ 'related_tax' ] ){
					$atts[ 'related_tax' ] = explode( ',', $atts[ 'related_tax' ] );
					foreach( $atts[ 'related_tax' ] as $tax ){
						$tax = trim( $tax );
						$args[ 'database' ][ 'post_type' ][ $post_type ][ 'taxonomy' ][ $tax ] = array( 'terms'=> array( ) );
						$terms = wp_get_post_terms( $post->ID, $tax );
						foreach( $terms as $term ){
							$term_id = $term->term_id;
							$args[ 'database' ][ 'post_type' ][ $post_type ][ 'taxonomy' ][ $tax ][ 'terms' ][ $term_id ] = array( );
						}
					}
					$query_args[ 'post__not_in' ] = array( $post->ID );
					$query_args[ 'orderby' ] = 'rand';
				}
			} // related posts over

			foreach($args['database']['post_type'] as $post_type_name => $post_type_obj){
				$query_args['post_type'][] = $post_type_name;

				foreach($post_type_obj['taxonomy'] as $taxonomy_name => $taxonomy_obj){

					$tax_arr = array(
						'taxonomy' => $taxonomy_name,
						'field'    => 'term_id',
						'terms'    => array(),
					);

					// add terms: user set term_ids in the shortcode
					foreach($taxonomy_obj['terms'] as $term_id => $term_obj){
						$tax_arr['terms'][] = $term_id;
					}

					$query_args['tax_query'][] = $tax_arr;
				}
			}
		}

		// filter to modify query
		if( ! empty( $atts[ 'id' ] ) ){
			$query_args = apply_filters( 'eap_query_args_'. $atts[ 'id' ], $query_args );
		}

		// perform query loop
		$query = new WP_Query( $query_args );
		global $post;
		ob_start( );
		$class = 'eap_temp_' . $this->count;
		?>
		<style>
			.<?php echo $class ?>{
				text-align: <?php echo $atts['align']; ?>;
			}

			.<?php echo $class ?> .eap_col{
				width: <?php echo 100/$args['resp'][ 'flex' ][0] ?>%;
			}

			@media (max-width:920px){
				.<?php echo $class ?> .eap_col{
					width: <?php echo 100/$args['resp'][ 'flex' ][1] ?>%;
				}
			}

			@media (max-width:720px){
				.<?php echo $class ?> .eap_col{
					width: <?php echo 100/$args['resp'][ 'flex' ][2] ?>%;
				}
			}
		</style>
		<?php
		echo '<div '. ( isset( $atts[ 'id' ] ) ? ' id="'.$atts[ 'id' ].'" ' : '' ) .' class="eap_grid '. $class .'">';
		if( $query->have_posts( ) ){
			while( $query->have_posts( ) ){
				$query->the_post( );

				echo '<div class="eap_col">';
					echo '<div class="eap_acc">';
					foreach( $args['cells'] as $cell ){
						$empty_indicator_class = '';
						if( $cell['type'] === 'Regular' ){
							if( empty( $cell['title'] ) ) $empty_indicator_class .= ' eap_no_title ';
							if( empty( $cell['content'] ) ) $empty_indicator_class .= ' eap_no_content ';
						}
						$pre_open_class = '';
						if( $cell['type'] === 'Regular' ){
							if( ! empty( $cell['pre_open'] ) ) $pre_open_class .= ' eap_open ';
						}

						$type_indicator_class = '';
						if( $cell['type'] !== 'Template' ){
							if( $cell['type'] === 'Featured Image' )
								$type_indicator_class = ' eap_type_image ';
							else {
								$type_indicator_class = ' eap_type_regular ';
							}
						}

						if( $cell['type'] !== 'Template' )
							echo '<div class="eap_cell '. $type_indicator_class . $empty_indicator_class . $pre_open_class .'">';

						switch ($cell['type']){
							case 'Regular':
								// process tags
								$tags = array( 'post_title', 'excerpt', 'content', 'featured_image', 'link', 'custom_field' );

								//-- cell title element
								if( ! empty( $cell['title'] ) ){
									foreach( $tags as $tag ){
										$cell['title'] = preg_replace_callback( '/(\(\(\s*'. $tag .'[^\)\)]+\)\))/', array( $this, 'replace_callback_' . $tag ), $cell['title'] );
									}
									echo '<div class="eap_title">' . $cell['title'] . '</div>';
								}

								//-- cell content element
								if( ! empty( $cell['content'] ) ){
									foreach( $tags as $tag ){
										$cell['content'] = preg_replace_callback( '/(\(\(\s*'. $tag .'[^\)\)]*\)\))/', array( $this, 'replace_callback_' . $tag ), $cell['content']);
									}
									echo '<div class="eap_content" style="'. ( $pre_open_class ? " display: block; " : "" ) .'">' . $cell['content'] . '</div>';
								}
								break;

							case 'Featured Image':
								the_post_thumbnail( $cell['image'] );
								break;

							case 'Template':
								$template_path = get_stylesheet_directory( ) .'/'. $cell['template'];
								if ( file_exists( $template_path ) ) { // searches in child theme and parent theme
									 include( $template_path );
								 } else if( file_exists( dirname( __FILE__ ) . '/templates/'. $cell['template'] ) ){
									 include( dirname( __FILE__ ) . '/templates/'. $cell['template'] );
								 } else if( current_user_can( 'manage_options' ) ) { // displays template error only to admins
									 echo '<div class="eap_cell eap_error">'.
									 	__('Template', 'easy-accordion-posts')
										.': <b>'. $cell['template'] .'</b>'. __(' does not exist! Check file name.', 'easy-accordion-posts') .'</div>';
								 }

								break;
						}
						if( $cell['type'] !== 'Template' )
							echo '</div>';
					}
					echo '</div>';
				echo '</div>';

			}
			if( ! empty( $args[ 'query' ][ 'pagination' ] ) ){
				echo "<div class='eap_pagination'>";
				$_count = $this->count;
				$pagination_args = array(
					// 'base'               => '%_%',
					// 'format'             => '?eappaged=%#%&eap_id='. $_count,
					'total'              => $query->max_num_pages,
					// 'show_all'           => false,
					// 'end_size'           => 1,
					// 'mid_size'           => 2,
					// 'prev_next'          => true,
					'prev_text'          => __('&#139;'),
					'next_text'          => __('&#155;'),
					// 'type'               => 'plain',
					// 'add_args'           => false,
					// 'add_fragment'       => '',
					// 'before_page_number' => '',
					// 'after_page_number'  => ''
				);

				if( isset( $atts[ 'id' ] ) ){
					$pagination_args[ 'format' ] = '?eap_paged_'.$atts[ 'id' ].'=%#%';
				}else{
					$pagination_args[ 'format' ] = '?eap_'.$_count.'_paged=%#%';
				}

				$pagination_args[ 'current' ] = 1;
				if( isset( $_GET['eap_'. $this->count .'_paged'] ) )
					$pagination_args[ 'current' ] = $_GET['eap_'. $this->count .'_paged'];
				else if( isset( $_GET[ 'eap_paged_'. $atts['id'] ] ) )
					$pagination_args[ 'current' ] = $_GET[ 'eap_paged_'. $atts['id'] ];

				echo paginate_links( $pagination_args );
				echo "</div>";
			}
		}else{
			echo '<h3 class="eap_no_results">'. $atts[ 'no_results' ] .'</h3>';
		}
		echo '</div>';
		wp_reset_postdata( );

		return ob_get_clean( );
	}

  // replaces post_title
  function replace_callback_post_title( $matches ){
    $defaults = array(
      'max_length'=> '85',
      'append'=> '...',
    );
    $title_args = $this->args_splitter( $matches[0], $defaults );
		$post_title = get_the_title( );

		// max length
		$post_title_exploded = str_split( $post_title );
		if( count( $post_title_exploded ) > (int)$title_args[ 'max_length' ] ){
			$post_title = substr( $post_title, 0, (int)$title_args[ 'max_length' ] );
			// append
			if( ! empty( $title_args[ 'append' ] ) ){
				$post_title .= $title_args[ 'append' ];
			}
		}
    return $post_title;
  }

  // replaces excerpt
  function replace_callback_excerpt( $matches ){

		// do not make wp create an excerpt as it applied 'the_content' filters while doing this,
		// which may lead to fatal recursion in case of 'Related Posts';
		global $post;
		if( has_excerpt( $post->ID ) )
			$excerpt = get_the_excerpt( );
		else{
			$excerpt = $this->replace_callback_content( $matches );
		}

    $defaults = array(
      'max_length'=> '200',
      'append'=> '...',
    );
    $excerpt_args = $this->args_splitter( $matches[0], $defaults );

		// max length
		$excerpt_exploded = str_split( $excerpt );
		if( count( $excerpt_exploded ) > ( int )$excerpt_args[ 'max_length' ] ){
			$excerpt = substr( $excerpt, 0, ( int )$excerpt_args[ 'max_length' ] );
			// append
			if( ! empty( $excerpt_args[ 'append' ] ) ){
				$excerpt .= $excerpt_args[ 'append' ];
			}
		}
    return $excerpt;

  }

	// replaces excerpt
  function replace_callback_content( $matches ){
    $defaults = array(
      'max_length'=> '5000',
      'append'=> '...',
			'strip_tags'=> false
    );
    $content_args = $this->args_splitter( $matches[0], $defaults );
		$content = get_the_content( );

		if($content_args['strip_tags'] !== false)
			$content = wp_strip_all_tags( $content );

		// max length
		$content_exploded = str_split( $content );
		if( count( $content_exploded ) > ( int )$content_args[ 'max_length' ] ){
			$content = substr( $content, 0, ( int )$content_args[ 'max_length' ] );
			// append
			if( ! empty( $content_args[ 'append' ] ) ){
				$content .= $content_args[ 'append' ];
			}
		}
    return $content;

  }

	// replaces link
  function replace_callback_featured_image( $matches ){
    $defaults = array(
      'size'=> 'Medium',
    );
    $link_args = $this->args_splitter( $matches[0], $defaults );
    return get_the_post_thumbnail();
  }

  // replaces link
  function replace_callback_link( $matches ){
    $defaults = array(
      'text'=> 'read more',
    );
    $link_args = $this->args_splitter( $matches[0], $defaults );
    return '<a href="'. get_the_permalink() .'">'. trim( $link_args['text'] ) .'</a>';
  }

	// replaces custom_field
  function replace_callback_custom_field( $matches ){
    $defaults = array(
      'max_length'=> '300',
      'append'=> '...',
			'key'=> ''
    );
    $meta_args = $this->args_splitter( $matches[ 0 ], $defaults );
		global $post;
		$meta_val = get_post_meta( $post->ID, trim($meta_args[ 'key' ]), true );

		// max length
		$meta_val_exploded = str_split( $meta_val );
		if( count( $meta_val_exploded ) > ( int )$meta_args[ 'max_length' ] ){
			$meta_val = substr( $meta_val, 0, ( int )$meta_args[ 'max_length' ] );
			// append
			if( ! empty( $meta_args[ 'append' ] ) ){
				$meta_val .= $meta_args[ 'append' ];
			}
		}
    return $meta_val;
  }

  // splits the arguments assigned to tags
  function args_splitter($string, $defaults){
    foreach( $defaults as $key=> &$val ){
      $matches = array( );
      $pattern = '/\|\s*'. $key .'\s*:([^\||^\)]+)\s*/';
      preg_match( $pattern, $string, $matches );
      if( ! empty( $matches ) ){
        $val = $matches[1];
      }
    }
    return $defaults;
  }

	/**
	 * Register the [eap_filter] shortcode
	 *
	 * @since    1.0.0
	 */
	public function filter_shortcode( $atts, $content = null ) {
		$atts = shortcode_atts( array(
				'label'=> __( 'Filters:', 'easy-accordion-posts' ),
				'all'=> __( 'All', 'easy-accordion-posts' ),
				'tax' => false,
				'id' => null,
		), $atts );

		if( ! $atts['tax'] || ! $atts['id'] ) return '<b>Your eap_filter shortcode needs these attributes: id, tax</b>';

		$permalink = get_the_permalink( );
		$markup = '';

		$selected = false;
		$taxes = explode( ',', $atts['tax'] );
		foreach( $taxes as $tax ){
			$tax = trim( $tax );
			$tax_obj = get_taxonomy( $tax );
			$tax_param = $atts['id'] .'_tax_'. $tax;
			$terms = get_terms( $tax );
			foreach( $terms as $term_obj ){

				if( $term_obj->parent ) continue; // only parent terms are wanted in this iteration

				$term_children = get_term_children( $term_obj->term_id, $tax );
				$has_children = false;

				if( count( $term_children ) ){
					$has_children = true;
				}

				$class_outer =  $has_children ? ' eap_parent_term ' : '';
				$class_inner = ( isset( $_GET[ $tax_param ] ) && ( $_GET[ $tax_param ] == $term_obj->term_id || in_array( $_GET[ $tax_param ], $term_children ) ) ) ? ' eap_active ' : '';

				if( $class_inner ) $selected = true;

				$markup .= '<div class="eap_term '. $class_outer .' "><a class="eap_term_link '. $class_inner .'" href="'. $permalink . '?' . $tax_param .'='. $term_obj->term_id .'">'. $term_obj->name .'</a>';

				if( $has_children ){
					$markup.= '<div class="eap_term_children">';
					$markup.= '<div class="eap_connector"></div>';

					foreach( $term_children as $child_term_id ){
						$child_term_obj = get_term( $child_term_id, $tax );

						$class_child = ( isset( $_GET[ $tax_param ] ) && ( $_GET[ $tax_param ] == $child_term_obj->term_id ) ) ? ' eap_active ' : '';
						if( $class_child ) $selected = true;
						$markup .= '<a class="eap_term_link '. $class_child .'" href="'. $permalink . '?' . $tax_param .'='. $child_term_obj->term_id .'">'. $child_term_obj->name .'</a>';

					}
					$markup.= '</div>';
				}

				$markup .= '</div>';

			}
		}

		$all_class = $selected ? '' : ' class="eap_active" ';

		$markup = '<div class="eap_filters" data-grid-id="'. $atts['id'] .'"><b class="eap_filters_label">'. $atts['label'] .'</b><div class="eap_term"><a '. $all_class .' class="eap_term_link" data-tax-param="'. $tax_param .'" href="'. $permalink .'">'. $atts[ 'all' ] .'</a></div>'. $markup .'</div>';

		return $markup;
	}

	/**
	 * Register the [eap_search] shortcode
	 *
	 * @since    1.0.0
	 */
	public function search_shortcode( $atts, $content = null ) {
		$atts = shortcode_atts( array(
				'id' => null,
				'label'=> __( 'Search:', 'easy-accordion-posts' ),
				'clear_search'=> __( 'Clear Search?', 'easy-accordion-posts' ),
				'submit'=> null,
		), $atts );

		$action = get_the_permalink();
		$append_action = '';

		// action params
		//-- preserve all vars except those related to id grd
		$hidden_fields = '';
		$prefix = $atts['id'] .'_tax_';
		$prefixlen = strlen( $prefix );
		$clear_link_append = '';
		foreach( $_GET as $key=> $val ){
			$test = substr( $key, 0, $prefixlen );
			if( $test !== $prefix && $key !== 'eap_search_'. $atts['id'] && $key !== 'eap_paged_'. $atts['id'] ){ // found a match : means we have a tax:term_id filter set for this grid
				$hidden_fields .= '<input type="hidden" name="'. $key .'" value="'. $val .'" />';
				$clear_link_append .= '&'. $key .'='. $val;
			}
		}

		$search_val = empty( $_GET[ 'eap_search_'. $atts['id'] ] ) ? '' : $_GET[ 'eap_search_'. $atts['id'] ];
		$hide_clear = empty( $_GET[ 'eap_search_'. $atts['id'] ] ) ? ' display: none; ' : '';

		$markup = '<form class="eap_search_form" method="GET" action="'. $action .'">
			<label for="eap_search_'. $atts[ 'id' ] .'"><b>'. $atts[ 'label' ] .'</b></label>
			<input id="eap_search_'. $atts[ 'id' ] .'" name="eap_search_'. $atts[ 'id' ] .'" type="text" value="'. $search_val .'" />';
		if( ! empty( $atts['submit'] ) ){
			$markup .= '<input class="eap_submit" type="submit" value="'. $atts['submit'] .'" />';
		};
		$markup .= $hidden_fields;

		if( $clear_link_append ){
			$clear_link_append = ltrim( $clear_link_append, '&' );
			$clear_link_append = '?'. $clear_link_append;
		}

		$markup .= '<a href="'. $action . $clear_link_append .'" style="'. $hide_clear .'">'. $atts[ 'clear_search' ] .'</a>';
		$markup .= '</form>';

		return $markup;
	}


}
